# -*- perl -*-

# Copyright (c) 2010 AdCopy
# Author: Jeff Weisberg
# Created: 2010-Jan-22 13:21 (EST)
# Function: debugging
#
# $Id: Debug.pm,v 1.1 2010/11/01 18:41:41 jaw Exp $

package AC::MrGamoo::Debug;
use AC::DC::Debug;
our @ISA = 'AC::DC::Debug';
use strict;

1;
