# -*- perl -*-

# Copyright (c) 2010 AdCopy
# Author: Jeff Weisberg
# Created: 2010-Apr-22 10:44 (EDT)
# Function: excitement!
#
# $Id: Info.pm,v 1.1 2010/11/01 18:41:56 jaw Exp $

package AC::MrGamoo::Job::Info;
use AC::MrGamoo::Debug 'job_info';
use AC::MrGamoo::Job::TaskInfo;
use AC::MrGamoo::Job::XferInfo;
use strict;


1;
